﻿/**********************************************************************************
 * Project Name :   
 * Remarks      :  
 * Class Name   :  UtilityCommon 
 * Version      :  v1.0.0.0 
 * Author       :  Joey Liang 
 * Description  : 
 * Created Time :  2013/9/2 20:51:33
 * Update Man   :  Joey Liang
 * Update Time  :  2013/9/2 20:51:33
************************************************************************/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IdleTest.DemoBeTest
{
    public class UtilityCommon
    {
        #region Foreach Handle
        /// <summary>
        /// 进行遍历
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <param name="array">遍历的集合</param>
        /// <param name="action">遍历到每一项执行的方法</param>
        /// <param name="breakBeforeFunc">跳出循环的方法(action执行前)</param>
        /// <param name="breakAfterFunc">跳出循环的方法(action执行后)</param>
        public static void ForeachHandle<T>(IEnumerable<T> array, Action<T> action,
            Func<T, bool> breakBeforeFunc, Func<T, bool> breakAfterFunc)
        {
            if (array == null || action == null)
            {
                return;
            }

            foreach (T item in array)
            {
                if (breakBeforeFunc != null && breakBeforeFunc(item))
                {
                    break;
                }

                action(item);
                if (breakAfterFunc != null && breakAfterFunc(item))
                {
                    break;
                }
            }
        }

        /// <summary>
        /// 进行遍历
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <param name="array">遍历的集合</param>
        /// <param name="action">遍历到每一项执行的方法</param>
        public static void ForeachHandle<T>(IEnumerable<T> array, Action<T> action)
        {
            ForeachHandle<T>(array, action, null, null);
        }

        /// <summary>
        /// 进行遍历,如果迭代器中的项不为T类型，则跳过不执行操作（action）
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <param name="array">遍历的集合</param>
        /// <param name="action">遍历到每一项执行的方法</param>
        /// <param name="breakBeforeFunc">跳出循环的方法(action执行前)</param>
        /// <param name="breakAfterFunc">跳出循环的方法(action执行后)</param>
        public static void ForeachHandle<T>(IEnumerable array, Action<T> action,
            Func<T, bool> breakBeforeFunc, Func<T, bool> breakAfterFunc)
        {
            if (array == null || action == null)
            {
                return;
            }

            foreach (var item in array)
            {
                if (item is T)
                {
                    T t = (T)item;
                    if (breakBeforeFunc != null && breakBeforeFunc(t))
                    {
                        break;
                    }

                    action(t);
                    if (breakAfterFunc != null && breakAfterFunc(t))
                    {
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// 进行遍历,如果迭代器中的项不为T类型，则不执行操作（action）
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <param name="array">遍历的集合</param>
        /// <param name="action">遍历到每一项执行的方法</param>
        public static void ForeachHandle<T>(IEnumerable array, Action<T> action)
        {
            ForeachHandle<T>(array, action, null, null);
        }
        #endregion
    }
}
