﻿/**********************************************************************************
 * Project Name :   
 * Remarks      :  
 * Class Name   :  AssertInstance 
 * Version      :  v1.0.0.0 
 * Author       :  Joey Liang 
 * Description  : 
 * Created Time :  2013/10/16 21:43:04
 * Update Man   :  Joey Liang
 * Update Time  :  2013/10/16 21:43:04
************************************************************************/

using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IdleTest.MSTest
{
    public class AssertInstance : IAssertInstance
    {
        #region IAssertInstance成员

        public virtual void AssertEqual<T>(T expected, T actual, bool areEqual = true)
        {
            if (!(actual is string) && !(expected is string) &&
                actual is IEnumerable && expected is IEnumerable && actual != null)
            {
                Assert.IsTrue(areEqual == TestCommon.CollectionEqual(expected, actual));
                return;
            }

            if (areEqual)
            {
                Assert.AreEqual<T>(expected, actual);
            }
            else
            {
                Assert.AreNotEqual<T>(expected, actual);
            }
        }

        public virtual void AssertIsNull<T>(T actual, bool assertNull = true)
        {
            if (assertNull)
            {
                Assert.IsNull(actual);
            }
            else
            {
                Assert.IsNotNull(actual);
            }
        }

        public virtual void AssertEmpty<T>(T actual, bool isEmpty = true)
        {
            if (actual is string)
            {
                Assert.IsTrue(isEmpty == string.IsNullOrEmpty(actual as string));
            }
            else if (actual is ICollection)
            {
                ICollection collection = actual as ICollection;
                Assert.IsTrue(isEmpty == (collection == null || collection.Count < 1));
            }
            else if (actual is ICollection<T>)
            {
                ICollection<T> collection = actual as ICollection<T>;
                Assert.IsTrue(isEmpty == (collection == null || collection.Count < 1));
            }
            else if (!(actual is ValueType))
            {
                AssertIsNull<T>(actual, isEmpty);
            }
            else
            {
                AssertFail("类型不支持断言Empty");
            }
        }

        public virtual void AssertIsInstance(Type expectedType, object actual, bool isInstance = true)
        {
            if (isInstance)
            {
                Assert.IsNotNull(actual);
                Assert.IsInstanceOfType(actual, expectedType);
            }
            else
            {
                Assert.IsNotInstanceOfType(actual, expectedType);
            }
        }

        public void AssertFail(string msg = "")
        {
            if (string.IsNullOrEmpty(msg))
            {
                Assert.Fail();
            }
            else
            {
                Assert.Fail(msg);
            }
        }
        #endregion

    }
}
