﻿/**********************************************************************************
 * Project Name   :   
 * Remarks        :  
 * Interface Name :  IAssertInstance   
 * Version        :  v1.0.0.0 
 * Author         :  Joey Liang 
 * Description    : 
 * Created Time   :  2013/10/16 21:38:37
 * Update Man     :  Joey Liang
 * Update Time    :  2013/10/16 21:38:37
************************************************************************/

using System;

namespace IdleTest
{
    /// <summary>
    /// 单元测试断言接口，用以实现测试框架的可替换性，当更换测试框架时，必须实现本接口，
    /// 然后修改AssertCommon类的DEFAULT_INSTANCE值为实现本接口的新类即可
    /// </summary>
    public interface IAssertInstance
    {
        /// <summary>
        /// 断言使用相等运算符验证指定的两个泛型数据是否相等。
        /// </summary>
        /// <typeparam name="T">类型参数</typeparam>
        /// <param name="expected">要比较的第一个泛型类型数据。 这是单元测试要求的泛型类型数据。</param>
        /// <param name="actual">要比较的第二个泛型类型数据。 这是单元测试生成的泛型类型数据。</param>
        /// <param name="areEqual">断言是否相等</param>
        void AssertEqual<T>(T expected, T actual, bool areEqual = true);

        /// <summary>
        /// 断言为null
        /// </summary>
        /// <param name="actual">需要断言的对象</param>
        /// <param name="assertNull">是否断言为空</param>
        void AssertIsNull<T>(T actual, bool assertNull = true);

        /// <summary>
        /// 断言为空或null
        /// </summary>
        /// <typeparam name="T">类型参数</typeparam>
        /// <param name="actual">实际值</param>
        /// <param name="isEmpty">是否断言为空</param>
        void AssertEmpty<T>(T actual, bool isEmpty = true);

        /// <summary>
        /// 断言是否是某一类型
        /// </summary>
        /// <param name="expectedType">预期类型</param>
        /// <param name="actual">需要断言的对象</param>
        /// <param name="isInstance">断言是否是expectedType类型</param>
        void AssertIsInstance(Type expectedType, object actual, bool isInstance = true);

        /// <summary>
        /// 断言失败
        /// </summary>
        /// <param name="msg">提示信息</param>
        void AssertFail(string msg = "");
    }
}
